﻿package
{ 
	import flash.display.*
	import flash.text.*
	import flash.events.*
	import flash.net.URLRequest;
	import flash.utils.*;
	
	public class Game extends MovieClip
	{
		//this static main property can be used to refer to this game object from anywhere easily using Game.main
 		static public var main;
		//The width and height of the game
		static public var WIDTH:int = 550;
		static public var HEIGHT:int = 400;
		
		//is theg ame muted?
		public var muted:Boolean;
		
		static public var backwardsTrays:Boolean = false;
		static public var maxDistance:int = 500;
		
		static public var score:Number = 0;
		
		/*	The current level.  Controls what appears in the level.
			Level 1:
			Snowmen should always be placed a set distance apart (in any direction), 
			such that two could be created rapidly, but their x coordinates would be very different, 
			or two could have the same x coordinate, but would be created some time appart, leaving their y coordinate different.
			Snowmen should be placed somewhat randomly, but not wildy. Some variation in frequency, but generally consistent.

			Level 2:
			Trees should be placed like snowmen, with distance from both other trees and snowmen.
			About as many trees as snowmen, but slightly more of both overall.
			Occaisional Time Warp trays, equally spaced throughout, with some noise. 
			Randomly placed, but always far enough from trees/snowmen to be picked up.

			Level 3:
			As level 2, plus slaloms about equally spaced throughout (again with noise), 
			such that they can be moved around without crashing relatively easily. 
			A tree or snowman could be placed adjacent, but only on the side that you don't need to go around.

			Level 4:
			As level 3, plus sledders placed such that they won't crash immediately. They can crash after a little bit though.
			Trays of Doom placed like Time Warp trays.

			Level 5:
			Sledders should be more common, but placed the same.
			Trays of Judgement added as other trays.

			Level 6:
			Add occasional random trays in place of other trays.

			Level 7+:
			More trees/snowmen, less powerups, etc.*/
		public var level:int;
		public var endLevel:int = 7;
		public var nextLevel:Boolean = false;
		
		//Width multiplier.  Higher numbers mean there's more space between objects.
		public const WIDE:int = 2;
		
		//Height of the tallest obstacle..
		public const TALLEST:int = 90;
		
		//Width of the widest obstacle.  Used so that obstacles aren't made outside the level.
		public const WIDEST:int = 70;
		
		//Distance value when the last powerup tray or obst was spawned.
		var lastTray:int;
		
		//Distance value when since last slalom was placed.
		var lastSlalom:int;
		
		//Distance value when the last obstacle (tree, snowman, sledder) was spawned.
		var lastDist:int;
		
		//X value of last object spawned.
		var lastX:int;
		
		//paused?
		static public var frozen:Boolean;
		
		//Start of the level? The game will be paused, and cannot be unpaused
		static public var starting:Boolean;
		
		//-1 = player exit, 0 = next level, 1 = game end
		static public var quitCondition:int = 0;
		
		//the difficulty level
		public var difficulty:int;
		
		//is the finish line placed?
		public var finish:Boolean;
		
		//the player
		public var player;
		
		//starting posts
		public var start1;
		public var start2;
		
		//the latest and oldest backgrounds
		public var latestBkg;
		public var oldBkg;
		
		//stuff to place
		public var placeThis;
		
		//slalom alternator: true is left, false is right
		public var slalomDirection:Boolean = true;
		
		//obstacles ready? This is important if distanceTraveled is not changing, so many trees do not get made at once
		public var ready:Boolean = true;
		
		//if this is false, the timer will not increment. Turned on and off each tick during Time Warp
		public var incrementTime:Boolean = true;
		
		//some useful variables
		public var distanceTraveled:Number;
		public var timeTraveled:Number;
		public var targetTime:Number;
		
		//text fields
		public var time:TextField = new TextField();
		public var goalTime:TextField = new TextField();
		public var currentScore:TextField = new TextField();
		public var levelText:TextField = new TextField();
		public var timeFormat:TextFormat = new TextFormat();
		public var goalFormat:TextFormat = new TextFormat();
		public var currentScoreFormat:TextFormat = new TextFormat();
		public var levelFormat:TextFormat = new TextFormat();
		
		//small display trays
		public var stealthDisplay;
		public var warpDisplay;
		public var judgeDisplay;
		public var doomDisplay;
		
		//the timer
		public var timer:Timer;
		
		public function Game(i)
		{
			/*	set the static main property to this game object so we can always just use "Game.main" 
			/	to refer to our game from other classes
			*/
			main = this;
			level = i;
			frozen = true;
			
			lastDist = 10;
			lastTray = 10;
			
			//text fields
			time = new TextField();
			goalTime = new TextField();
			currentScore = new TextField();
			levelText = new TextField();
			timeFormat = new TextFormat();
			goalFormat = new TextFormat();
			levelFormat = new TextFormat();
			
			this.addEventListener(Event.ADDED_TO_STAGE, added);
		}
				
		public function added(e:Event)
		{
			Key.initialize(stage);
			
			//Set a timer to go off after 1 second that triggers spawning code.
			timer = new Timer(100, 0);
			
			initGame();
		}
		
		//event for key press
		public function keyDownHandler(e:KeyboardEvent)
		{
			if ((e.keyCode == 80) && (!starting))
			{
				frozen = !frozen;
			}
		}
		
		//event for when a key is released
		public function keyUpHandler(e:KeyboardEvent)
		{
			if (e.keyCode == 81)
			{
				quitCondition = -1;
				quit();
			}
		}
		
		public function timeUpdate(e:TimerEvent)
		{
			if (player.parent != null)
			{
				player.focusRect = false;
				stage.focus = player;
			}
			
			if (!frozen)
			{
				if (player.timeWarp)
				{
					incrementTime = !incrementTime;
				}
				else
				{
					incrementTime = true;
				}
				
				if (incrementTime)
				{
					timeTraveled += 1;
				}
				
				updateText();
				
				if (player.moving)
				{
					ready = true;
					distanceTraveled += 1;
				}
				
				resetX();
				
				if (distanceTraveled < maxDistance - 20)
				{
					if (level > 2) placeSlaloms();
					
					placeObstacles();
					
					placePowerUps();
				}
				
				ready = false;
				
				if (distanceTraveled >= maxDistance && !finish)
				{
					finish = true;
					placeFinishLine();
				}
				
				//make sure the trail is above the background
				var i:int = parent.numChildren;
				
				while(i--)
				{
					var obj:DisplayObject = parent.getChildAt(i);
					if (obj is Trail)
					{
						setChildIndex(obj, getChildIndex(latestBkg) + 1);
					}
				}
			}
		}
		
		public function updateText()
		{
			timeFormat.font = "Comic Sans MS";
			timeFormat.bold = true;
			timeFormat.size = 12;
			timeFormat.color = "0x8a162c";
			timeFormat.align = TextFormatAlign.CENTER;
			
			goalFormat.font = "Comic Sans MS";
			goalFormat.bold = true;
			goalFormat.size = 12;
			goalFormat.color = "0x8a162c";
			goalFormat.align = TextFormatAlign.CENTER;
			
			currentScoreFormat.font = "Comic Sans MS";
			currentScoreFormat.bold = true;
			currentScoreFormat.size = 12;
			currentScoreFormat.color = "0x8a162c";
			currentScoreFormat.align = TextFormatAlign.CENTER;
			
			levelFormat.font = "Comic Sans MS";
			levelFormat.bold = true;
			levelFormat.size = 16;
			levelFormat.color = "0x8a162c";
			levelFormat.align = TextFormatAlign.CENTER;
			
			time.height = 400;
			time.width = 400;
			time.x = 250;
			time.y = 40;
			time.selectable = false;
			
			goalTime.width = 400;
			goalTime.height = 400;
			goalTime.x = 250;
			goalTime.y = 60;
			goalTime.selectable = false;
			
			currentScore.height = 400;
			currentScore.width = 400;
			currentScore.x = 250;
			currentScore.y = 20;
			currentScore.selectable = false;
			
			levelText.height = 100;
			levelText.width = 100;
			levelText.x = 0;
			levelText.y = 20;
			levelText.selectable = false;
			
			goalTime.text = "Time: " + timeTraveled/10;
			time.text = "Goal Time: " + targetTime/10;
			currentScore.text = "Score: " + score;
			levelText.text = "Level: " + level;
			
			time.setTextFormat(timeFormat);
			goalTime.setTextFormat(goalFormat);
			currentScore.setTextFormat(currentScoreFormat);
			levelText.setTextFormat(levelFormat);
		}
		
		public function placeSlaloms()
		{
			var r:int = ((Math.random() * difficulty) + 1);
			//If 300*difficulty miliseconds have passed since the last slalom was placed, try to place a new one.
			if ( ( distanceTraveled - 3 * difficulty ) > lastSlalom )
			{
				if (ready && r == 1)
				{
					if (slalomDirection)
					{
						r = ((Math.random() * 100) + 100);
						placeThis = new SlalomL(r, HEIGHT + 40);
						this.addChild(placeThis);
						this.setChildIndex(placeThis, 4);
						slalomDirection = false;
						lastSlalom = distanceTraveled;
						lastDist = distanceTraveled;
						lastX = r;
					}
					else
					{
						r = (WIDTH - 100 - (Math.random() * 100));
						placeThis = new SlalomR(r, HEIGHT + 40);
						this.addChild(placeThis);
						this.setChildIndex(placeThis, 4);
						slalomDirection = true;
						lastSlalom = distanceTraveled;
						lastDist = distanceTraveled;
						lastX = r;
					}
					ready = false;
				}
			}
		}
		
		public function placeObstacles()
		{
			var r:int = ((Math.random() * 20/difficulty) + 1);
			if (ready && r == 1  && ( ( distanceTraveled - 4 ) > lastDist ) )
			{
				//Each case is a different level
				switch(level){
					case 1:
						//Level 1, only spawns snowmen.
						placeSnowman();
						break;
					case 2:
						r = Math.random() * 2 + 1;
						//Level 2, spawn snowmen or trees.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
						}
						break;
					case 3:
						r = Math.random() * 2 + 1;
						//Level 3, same as level 2.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
						}
						break;
					case 4:
						r = Math.random() * 3 + 1;
						//Level 4, add sledders.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
							case 3:
								placeSledder();
								break;
						}
						break;
					case 5:
						r = Math.random() * 4 + 1;
						//Level 5, add more sledders.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
							case 3:
								placeSledder();
								break;
							case 4:
								placeSledder();
								break;
						}
						break;
					case 6:
						r = Math.random() * 4 + 1;
						//Level 6, same as 5.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
							case 3:
								placeSledder();
								break;
							case 4:
								placeSledder();
								break;
						}
						break;
					case 7:
						r = Math.random() * 5 + 1;
						//Level 7, same as 6.
						switch(r){
							case 1:
								placeSnowman();
								break;
							case 2:
								placeTree();
								break;
							case 3:
								placeSledder();
								break;
							case 4:
								placeSnowman();
								break;
							case 5:
								placeTree();
								break;
						}
						break;
				}
				
				ready = false;
			}
		}
		
		public function placeTree()
		{
			var r:int = findX();
			placeThis = new Plant(r, HEIGHT + 40);
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 4);
			lastDist = distanceTraveled;
			lastX = r;
		}
		
		private function placeSledder()
		{
			var r:int = findX();
			while( (r > WIDTH/4) && (r < (WIDTH - WIDTH/4)) )
			{
				r = findX();
			}
			
			if (r < 20) r = 20;
			if (r > WIDTH - 20) r = WIDTH - 20;
			placeThis = new Sledder(r, HEIGHT + 40);
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 2);
			lastDist = distanceTraveled;
			lastX = r;
		}
		
		//Find a place to put a snowman
		private function placeSnowman()
		{
			var r:int = findX();
			placeThis = new SnowMan(r, HEIGHT + 40);
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 4);
			lastDist = distanceTraveled;
			lastX = r;
		}
		
		private function placePowerUps()
		{
			var yLoc:Number;
			if (backwardsTrays) yLoc = -40;
			else yLoc = HEIGHT + 40;
			
			var r:int = ((Math.random() * 8 * difficulty) + 1);
			if (ready && r == 1 && ( ( distanceTraveled - 20 ) > lastTray ) )
			{
				//Each level is a different case
				switch(level){
					case 1:
						r = findX();
						placeThis = new TrayOfStealth(r, yLoc)
						this.addChild(placeThis);
						this.setChildIndex(placeThis, 4);
						lastX = r;
						lastTray = distanceTraveled;
						break;
					case 2:
						r = ((Math.random() * 2) + 1);
						if (r == 1)
						{
							r = findX();
							placeThis = new TrayOfStealth(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						break;
					case 3:
						r = ((Math.random() * 2) + 1);
						if (r == 1)
						{
							r = findX();
							placeThis = new TrayOfStealth(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						break;
					case 4:
						r = ((Math.random() * 3) + 1);
						if (r == 1)
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 2)
						{
							r = findX();
							placeThis = new TrayOfStealth(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else
						{
							r = findX();
							placeThis = new TrayOfDoom(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						break;
					case 5:
						r = ((Math.random() * 5) + 1);
						if (r == 1)
						{
							r = findX();
							placeThis = new TrayOfStealth(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 2)
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 3)
						{
							r = findX();
							placeThis = new TrayOfDoom(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else
						{
							r = findX();
							placeThis = new TrayOfJudgement(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						break;
					default:
						r = ((Math.random() * 6) + 1);
						if (r == 1)
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 2)
						{
							r = findX();
							placeThis = new TrayOfTimeWarp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 3)
						{
							r = findX();
							placeThis = new TrayOfDoom(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else if (r == 4)
						{
							r = findX();
							placeThis = new TrayOfJudgement(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;
							lastTray = distanceTraveled;
						}
						else
						{
							r = findX();
							placeThis = new PowerUp(r, yLoc)
							this.addChild(placeThis);
							this.setChildIndex(placeThis, 4);
							lastX = r;	
							lastTray = distanceTraveled;		
						}
						break;
				}
				
				ready = false;
			}
		}
		
		
		
		//Find an appropriate x value to spawn an obstacle or tray at.
		private function findX():Number
		{
			//Find an x value that exists on the stage.
			var r:int = ((Math.random() * (WIDTH - WIDEST)));
			//If the x value generated is within 24*WIDE pixels, find a new one.
			while( ((r - lastX) < (WIDEST * WIDE) && (lastX - r) < (WIDEST * WIDE)) || r < 35)
			{
				r = ((Math.random() * (WIDTH - WIDEST)));
			}
				
			return r;
		}
		
		//Resets the value of lastX if enough time has passed.
		private function resetX()
		{
			if( (distanceTraveled - lastDist) > (3 * TALLEST) )
			{
				lastX = 1000;
			}
		}
		
		public function placeFinishLine()
		{
			placeThis = new FinishLine(WIDTH/2, HEIGHT + 10)
			this.addChild(placeThis);
		}
		
		//when the game is started
		public function initGame()
		{
			timer.addEventListener(TimerEvent.TIMER, timeUpdate);
			
			//add some listeners for key presses
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUpHandler);
			
			finish = false;
			
			frozen = true;
			starting = true;
			
			distanceTraveled = 0;
			timeTraveled = 0;
			
			lastSlalom = 0;
			
			switch (level)
			{
				case 1:
					targetTime = 600;
					break;
				case 2:
					targetTime = 550;
					break;
				case 3:
					targetTime = 500;
					break;
				case 4:
					targetTime = 500;
					break;
				case 5:
					targetTime = 450;
					break;
				case 6:
					targetTime = 450;
					break;
				default:
					targetTime = 400;
					break;
			}
			
			//the player
			player = new Player(WIDTH/2, HEIGHT/2);
		
			//starting posts
			start1 = new StartPole(WIDTH/2 - 30, HEIGHT/2);
			start2 = new StartPole(WIDTH/2 + 30, HEIGHT/2);
			
			//display the background
			placeThis = new Bkg(0, 0);
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 0)
			oldBkg = placeThis;
			
			placeThis = new Bkg(0, 0);
			placeThis.y = 1200;
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 1)
			latestBkg = placeThis;

			placeThis = new Trail(player.x, player.y + 20);
			this.addChild(placeThis);
			this.setChildIndex(placeThis, 2);
			
			//display the player
			this.addChild(player);
			this.setChildIndex(player, 3);

			//display the starting poles
			this.addChild(start1);
			this.setChildIndex(start1, 4);
			this.addChild(start2);
			this.setChildIndex(start2, 4);
			
			this.addChild(time);
			this.addChild(goalTime);
			this.addChild(currentScore);
			this.addChild(levelText);
			updateText();
			
			stealthDisplay = new TrayOfStealthDisplay();
			stealthDisplay.x = 150;
			stealthDisplay.y = 20;
			stealthDisplay.alpha = 0.5;
			warpDisplay = new TrayOfTimeWarpDisplay();
			warpDisplay.x = 200;
			warpDisplay.y = 20;
			warpDisplay.alpha = 0.5;
			judgeDisplay = new TrayOfJudgementDisplay();
			judgeDisplay.x = 300;
			judgeDisplay.y = 20;
			judgeDisplay.alpha = 0.5;
			doomDisplay = new TrayOfDoomDisplay();
			doomDisplay.x = 250;
			doomDisplay.y = 20;
			doomDisplay.alpha = 0.5;
			
			this.addChild(stealthDisplay);
			this.addChild(warpDisplay);
			this.addChild(judgeDisplay);
			this.addChild(doomDisplay);
			
			placeThis = new RedLight(WIDTH/2, HEIGHT/2 - 50);
			this.addChild(placeThis);
		}
		
		public function quit()
		{			
			var i:int = numChildren;

			var that;
			
			while(i--)
				{ 
					var obj:DisplayObject = getChildAt(i);
					if (obj is Killable)
					{
						that = obj as Killable;
						that.kill();
					}
				}
				
			this.removeChild(time);
			this.removeChild(goalTime);
			this.removeChild(currentScore);
			this.removeChild(levelText);
			this.removeChild(stealthDisplay);
			this.removeChild(warpDisplay);
			this.removeChild(judgeDisplay);
			this.removeChild(doomDisplay);
			
			timer.removeEventListener(TimerEvent.TIMER, timeUpdate);
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
			stage.removeEventListener(KeyboardEvent.KEY_UP, keyUpHandler);
			frozen = true;
			starting = true;
			lastDist = 10;
			lastTray = 10;
			
			if (quitCondition == 0)
			{
				nextLevel = true;
			}
		}
	}
}